
package org.bug.glassfish.servlet;

import java.io.IOException;

import javax.inject.Inject;
import javax.servlet.AsyncContext;
import javax.servlet.AsyncEvent;
import javax.servlet.AsyncListener;
import javax.servlet.DispatcherType;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.bug.glassfish.RequestBean;

@WebServlet(name = "Async Servlet", urlPatterns = "/test", asyncSupported = true)
public class AsyncServlet extends HttpServlet implements AsyncListener {

    private static final long serialVersionUID = 1L;

    @Inject
    private RequestBean requestBean;

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        doPost(request, response);
    }

    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

        if (request.getDispatcherType() != DispatcherType.ASYNC) {
            this.requestBean.setName("Bob");

            final AsyncContext context = request.startAsync();

            context.addListener(this);
            
            context.start(new Runnable() {
                public void run() {
                    try {
                        Thread.sleep(3000);
                        context.getRequest().setAttribute("Status", "Done");
                        context.dispatch();
                    } catch (InterruptedException e) {}
                }
            });
        } else if (request.getAttribute("Status").equals("Done")) {
            System.err.println(this.requestBean.getName());
        } else {
            throw new ServletException("Unexpected state");
        }
    }

    public void onComplete(AsyncEvent event) throws IOException {
        System.err.println("Async Complete");
    }

    public void onError(AsyncEvent event) throws IOException {        
        final AsyncContext context = event.getAsyncContext();
        
        try {
            event.getThrowable().printStackTrace(context.getResponse().getWriter());
        } finally {
            context.complete();
        }
    }

    public void onStartAsync(AsyncEvent event) throws IOException {}

    public void onTimeout(AsyncEvent event) throws IOException {
        System.err.println("Async Timeout");
    }
}
